/*
    ettercap -- module for NON ncurses interface

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <signal.h>

#include "ec_main.h"
#include "ec_error.h"
#include "ec_inet.h"
#include "ec_decodedata.h"
#include "ec_illithid.h"
#include "ec_doppleganger.h"

#ifdef DEBUG
	#include "ec_debug.h"
#endif

#ifdef HAVE_TERMIOS_H

	#include <termios.h>

	struct termios old_tc;
	struct termios new_tc;

	void set_raw_term(void);
	void reset_term(void);

#endif

#define ASCII_VIEW   0
#define HEX_VIEW		1

// protos...

void Simple_HostList(void);
void Simple_Run(void);

//---------------------------------

void Simple_HostList(void)
{
	int j;

#ifdef DEBUG
	Debug_msg("Simple_HostList");
#endif

	for(j=0; j<number_of_hosts_in_lan; j++)
	{
		printf("%3d) %s\n", j+1, Host_In_LAN[j].ip);
	}

	printf("\n");
}



#ifdef HAVE_TERMIOS_H

void set_raw_term()			// taken from readchar.c, by M. Andreoli (2000)
{
    tcgetattr(0, &old_tc);
    new_tc = old_tc;
    new_tc.c_lflag &= ~(ECHO | ICANON);	/* raw output */
    new_tc.c_cc[VTIME] = 1;

    tcsetattr(0, TCSANOW, &new_tc);
}

void reset_term()				// taken from readchar.c, by M. Andreoli (2000)
{
    tcsetattr(0, TCSANOW, &old_tc);
}

#endif



void Simple_Run(void)
{
	int Illithid_pid = 0, Dopple_pid = 0;
	int i, index_source=-1, index_dest=-1;
	char view = ASCII_VIEW;
	char stop = 0;
	char SourceIP[16];
	char DestIP[16];
	fd_set msk_fd;
	struct timeval TimeOut;

#ifdef DEBUG
	Debug_msg("Simple_Run -- %s <--> %s", Options.con_source, Options.con_dest);
#endif

	if (strcmp(Options.con_source, ""))
		strcpy(SourceIP, Inet_NameToIp(Options.con_source));
	else
		strcpy(SourceIP, "");
	if (strcmp(Options.con_dest, ""))
		strcpy(DestIP, Inet_NameToIp(Options.con_dest));
	else
		strcpy(DestIP, "");

	for(i=0; i<number_of_hosts_in_lan; i++)
	{
		if ( !strcmp(SourceIP, Host_In_LAN[i].ip) ) index_source = i;
		if ( !strcmp(DestIP, Host_In_LAN[i].ip) ) index_dest = i;
	}


	if ( index_source < 0 && strcmp(Options.con_source, ""))
		Error_msg("Source host %s (%s) not found !!", Options.con_source, SourceIP);


	if ( index_dest < 0 && strcmp(Options.con_dest, ""))
		Error_msg("Dest host %s (%s) not found !!", Options.con_dest, DestIP);

	if (!strcmp(SourceIP, DestIP) && strcmp(Options.con_source, ""))
		Error_msg("Source and Dest MUST be different !!");

	if (Options.connect == 1)
		if ( (!strcmp(SourceIP, Inet_MyIPAddress())) || (!strcmp(DestIP, Inet_MyIPAddress())) )
			Error_msg("CAN'T connect yourself !!");

	// it is ok...

	if (Options.connect == 1)		// Doppleganger born...
		Dopple_pid = Doppleganger_Run(Options.netiface, Host_In_LAN[index_source].ip, Host_In_LAN[index_dest].ip, Host_In_LAN[index_source].mac, Host_In_LAN[index_dest].mac);

	printf("\nPress 'h' for help...\n\n");

	pipe(pipe_with_illithid_data);


	if (Options.connect == 1)
	{
		printf(" Connected : %s:%d <--> %s <--> %s:%d\n\n", SourceIP, Options.port_source, Inet_MyIPAddress(), DestIP, Options.port_dest);
		Illithid_pid = Illithid_ARPBased_GetData(Options.netiface, Host_In_LAN[index_source].ip, Options.port_source, Host_In_LAN[index_dest].ip, Options.port_dest, Host_In_LAN[index_source].mac, Host_In_LAN[index_dest].mac);
	}
	else if (Options.sniff == 1)
	{
		printf(" Sniffing (IP based): %s:%d <--> %s:%d\n\n", SourceIP, Options.port_source, DestIP, Options.port_dest );
		Illithid_pid = Illithid_IPBased_GetData(Options.netiface, SourceIP, Options.port_source, DestIP, Options.port_dest);
	}
	else if (Options.macsniff == 1)
	{
		printf(" Sniffing (MAC based): %s <--> %s\n\n", SourceIP, DestIP);
		Illithid_pid = Illithid_MACBased_GetData(Options.netiface, Host_In_LAN[index_source].mac, Host_In_LAN[index_dest].mac);
	}



		memset(&TimeOut, 0, sizeof(TimeOut));	//  timeout = 0
		FD_ZERO(&msk_fd);


#ifdef HAVE_TERMIOS_H
		set_raw_term();	// non blocking stdin... yes this work !!
#else
	#ifdef DEBUG
		Debug_msg("Simple_Run -- NO TERMIOS_H");
	#endif
		fcntl(0, F_SETFL, O_NONBLOCK);	// stdin non blocking... seems to be a non working method for me...
#endif

		loop
		{

			FD_SET(pipe_with_illithid_data[0], &msk_fd);
			FD_SET(0, &msk_fd);

			select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);

			if (FD_ISSET(0, &msk_fd))
			{
				char ch = 0;
				ch = getchar();
				switch(ch)
				{
					case 'A':
					case 'a':
							printf("\n\nAscii mode...\n\n");
							view = ASCII_VIEW;
							break;

					case 'X':
					case 'x':
							printf("\n\nHex mode...\n\n");
							view = HEX_VIEW;
							break;

					case 'S':
					case 's':
							if (stop == 1)
							{
								printf("\n\nLet's go...\n\n");
								stop = 0;
							}
							else
							{
								printf("\n\nStopped...\n\n");
								stop = 1;
							}
							break;

					case 'H':
					case 'h':
							printf("\n\n[qQ]  - quit\n");
							printf(    "[aA]  - dump data in ascii mode\n");
							printf(    "[xX]  - dump data in hex mode\n");
							printf(    "[sS]  - stop/cont sniffing\n\n");
							break;

					case 'Q':
					case 'q':
#ifdef HAVE_TERMIOS_H
							reset_term();
#endif
							printf("\n\n\nQuitting...\n\n");
							kill(Illithid_pid, SIGTERM);
							if (Dopple_pid) kill(Dopple_pid, SIGTERM);
							exit(0);
							break;
				}
			}

			if (FD_ISSET(pipe_with_illithid_data[0], &msk_fd))
			{
					SNIFFED_DATA data_from_illithid;

					memset(&data_from_illithid, 0, sizeof(SNIFFED_DATA));
					read(pipe_with_illithid_data[0], &data_from_illithid, sizeof(SNIFFED_DATA));


					if (!stop && data_from_illithid.datasize != 0)
						switch (view)
						{
							case ASCII_VIEW:
								printf("\n\n%s:%d -> %s:%d\n%s",
									data_from_illithid.ip_source,
									data_from_illithid.port_source,
									data_from_illithid.ip_dest,
									data_from_illithid.port_dest,
									Decodedata_GetAsciiData(data_from_illithid.data, data_from_illithid.datasize)
								);
								break;

							case HEX_VIEW:
								printf("\n\n%s:%d -> %s:%d\n%s",
									data_from_illithid.ip_source,
									data_from_illithid.port_source,
									data_from_illithid.ip_dest,
									data_from_illithid.port_dest,
									Decodedata_GetHexData(data_from_illithid.data, data_from_illithid.datasize, 80)
								);
								break;
						}
					fflush(stdout);
			}
			else
				usleep(1);

		}



}

