/*
    ettercap -- inet utilities, arp ping and more...

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>


#include "include/ec_main.h"
#include "include/ec_error.h"
#include "include/ec_inet_structures.h"
#include "include/ec_inet_forge.h"

#ifdef DEBUG
   #include "include/ec_debug.h"
#endif


char ETH_BROADCAST[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
char ARP_BROADCAST[6] = {0x0,0x0,0x0,0x0,0x0,0x0};

typedef struct {
   u_long IP_Add;
   u_char MAC_Add[6];
   struct host_list *next;
} host_list;

struct ifreq old_ifr;      // old iface flags
char IpForward_status[2];  // old ipforward status

// protos...

char * Inet_HostName(char *ip);
char * Inet_NameToIp(char *name);
char * Inet_GetMyInfo(char tipo);
char * Inet_MyIPAddress(void);
char * Inet_MyMACAddress(void);
char * Inet_MySubnet(void);
int Inet_HostInLAN(void);
void Inet_Free_list(host_list *head);
host_list *Inet_Host_in_LAN_list(char *interface);
SniffingHost *Inet_NoSniff(void);
void Inet_PutMACinString(char *mac_string, unsigned char *MAC);
void Inet_GetMACfromString(char *mac_string, unsigned char *MAC);
// Following are architecture dependent !! implementations are in ./src/`uname`/ec_inet_`uname`.c
int Inet_FindIFace(char *iface);
char Inet_CorrectIface(char *iface);
int Inet_GetIfaceInfo(char *iface, int *MTU, int *ifnumber, char *MyMAC, u_long *IP, u_long *NetMask);
int Inet_SetPromisc(char *iface);
int Inet_OpenRawSock(char *iface);
int Inet_GetRawPacket(int sock, char *buffer, int MTU, short *type);
int Inet_SendRawPacket(int sock, char *buffer, int len);
void Inet_Restore_ifr(void);
void Inet_DisableForwarding(void);
void Inet_RestoreForwarding(void);
char *Inet_MacFromIP(unsigned long ip);

// ----------------------------------------



#ifdef LINUX
   #include "linux/ec_inet_linux.c"    // Architecture dependent implemetation of Inet functions...
#endif




char * Inet_HostName(char *ip)   // returns hostname from ip
{
   struct hostent *host;
   static struct in_addr addr;

   addr.s_addr = inet_addr(ip);
   host = gethostbyaddr((char *)&addr, sizeof(struct in_addr), AF_INET);

   if (host != NULL) return (char *) host->h_name;

   return "Unknown host";
}


char * Inet_NameToIp(char *name) // returns ip from hostname
{
   struct hostent *host;
   static char ip[16];

   host = gethostbyname(name);

   if (host != NULL)
      strncpy(ip, inet_ntoa( *(struct in_addr *) *host->h_addr_list), 16);
   else
      strncpy(ip, "(null)", 16);

   return ip;
}


char * Inet_GetMyInfo(char tipo)
{
   struct in_addr  to_convert;
   u_long IP_Add, NetMask;
   unsigned char MAC_Add[6];
   static char MAC[18];
   static char MySubnet[16];
   static char IP[16];

#ifdef DEBUG
   switch (tipo)
   {
      case 0: Debug_msg("Inet_GetMyInfo IP"); break;
      case 1: Debug_msg("Inet_GetMyInfo MAC"); break;
      case 2: Debug_msg("Inet_GetMyInfo NetMask"); break;
   }
#endif

   Inet_GetIfaceInfo(Options.netiface, NULL, NULL, MAC_Add, &IP_Add, &NetMask);


   switch (tipo)
   {
         case 0:  // requesting IP
                  to_convert.s_addr = IP_Add;
                  sprintf(IP, "%s", inet_ntoa(to_convert));
                  return IP;
                  break;

         case 1:  // requesting MAC
                  Inet_PutMACinString( MAC, MAC_Add );
                  return MAC;
                  break;

         case 2:  // requesting netmask
                  to_convert.s_addr = NetMask;
                  sprintf(MySubnet, "%s", inet_ntoa(to_convert));
                  return MySubnet;
                  break;
   }

   return "(none)";

}



char * Inet_MyIPAddress(void)
{
   return Inet_GetMyInfo(0);
}



char * Inet_MyMACAddress(void)
{
   return Inet_GetMyInfo(1);
}



char * Inet_MySubnet(void)
{
   return Inet_GetMyInfo(2);
}




int Inet_HostInLAN()
{
   host_list *list;
   host_list *index;
   int host_alive = 0, k = 0;

#ifdef DEBUG
   Debug_msg("Inet_HostInLAN\tIface: %s", Options.netiface);
#endif

   list = Inet_Host_in_LAN_list(Options.netiface);

   index = list;

   for( ; index; host_alive++)
      index = (host_list *)index->next;

   if (Host_In_LAN) free(Host_In_LAN);
   Host_In_LAN = (HOST *)calloc(host_alive,sizeof(HOST));
   if (Host_In_LAN == NULL)
      Error_msg("ec_Inet:%d calloc() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);

   index = list;
   for( ; index; index = (host_list *)index->next)
   {
      struct in_addr to_convert;
      to_convert.s_addr = index->IP_Add;
      sprintf(Host_In_LAN[k].ip,"%s",inet_ntoa(to_convert));
      snprintf(Host_In_LAN[k].name, 128, "%s", Inet_HostName(Host_In_LAN[k].ip));
      Inet_PutMACinString(Host_In_LAN[k].mac, index->MAC_Add);
      k++;
   }
   Inet_Free_list(list);

   return host_alive;
}




void Inet_Free_list(host_list *head)
{
   if (!head) return;

   Inet_Free_list((host_list *) head->next);
   free(head);
}



host_list *Inet_Host_in_LAN_list(char *interface)
{
   int sock, N_hosts, index, MTU;
   u_long NetMask;
   host_list *head;
   u_char *buf;
   TIME_DECLARE;


#ifdef DEBUG
   Debug_msg("Inet_HostInLAN_list");
#endif

   head = (host_list *)malloc(sizeof(host_list));
   if (head == NULL)
         Error_msg("ec_Inet:%d malloc() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);
   head->next=NULL;

   sock = Inet_OpenRawSock(interface);

   Inet_GetIfaceInfo(interface, &MTU, NULL, head->MAC_Add, &head->IP_Add, &NetMask);

   N_hosts = ntohl(~NetMask);

   buf = Inet_Forge_packet( ETH_HEADER + ARP_HEADER );

   // frame ethernet header
   Inet_Forge_ethernet( buf, head->MAC_Add, ETH_BROADCAST, ETH_P_ARP );

   for (index=1; index<=N_hosts; index++)
   {
      int dest_ip;
      dest_ip = (head->IP_Add&NetMask)|htonl(index);

      // if dest is equal to me
      if (dest_ip != head->IP_Add)
      {
         // arp request
         Inet_Forge_arp( buf+ETH_HEADER, ARPOP_REQUEST,
                         head->MAC_Add, head->IP_Add,
                         ARP_BROADCAST, dest_ip );

         Inet_SendRawPacket(sock, buf, ETH_HEADER + ARP_HEADER);
      }
      usleep(500);
   }

   Inet_Forge_packet_destroy( buf );

#ifdef DEBUG
   Debug_msg("Inet_HostInLAN_list -- listening for replies...");
#endif

   fcntl(sock, F_SETFL, O_NONBLOCK);

   TIME_START;

   buf = Inet_Forge_packet( MTU );

   // wait for reply
   do
   {
      int leng;
      short pkttype;
      host_list **index;
      ETH_header *ethpkt;
      ARP_header *arppkt;

      leng = Inet_GetRawPacket(sock, buf, MTU, &pkttype);

      ethpkt = (ETH_header *)buf;
      arppkt = (ARP_header *)(buf + ETH_HEADER);

      TIME_FINISH;

      if (leng > 0 && pkttype == PACKET_HOST && ethpkt->type == htons(ETH_P_ARP) && arppkt->opcode == htons(ARPOP_REPLY))
      {
         index = &head;

#ifdef DEBUG
   Debug_msg("Inet_HostInLAN_list -- got a reply after %.5f seconds", TIME_ELAPSED );
#endif
         while(*index != NULL && memcmp(&((*index)->IP_Add),arppkt->source_ip,4))
            index = (host_list **)&((*index)->next);

         if (*index == NULL)
         {
            *index = (host_list *)malloc(sizeof(host_list));
            (*index)->next = NULL;
            memcpy((*index)->MAC_Add, arppkt->source_add, 6);
            memcpy((char *)&((*index)->IP_Add), arppkt->source_ip, 4);
         }
      }
   } while ( TIME_ELAPSED < 1.5 );


   Inet_Forge_packet_destroy( buf );
   close(sock);
   return head;
}



SniffingHost *Inet_NoSniff(void)
{
   static SniffingHost *SniffTable=NULL;
   int i, j, len, sock, MTU, SniffTableIndex=0;
   ETH_header  *HEther;
   IP_header *HIP;
   u_char *buf;
   TIME_DECLARE;

#ifdef DEBUG
   Debug_msg("Inet_NoSniff");
#endif

   if (SniffTable) free (SniffTable);
   SniffTable = calloc(number_of_hosts_in_lan, sizeof(SniffingHost));
   memset(SniffTable,0,sizeof(SniffingHost)*number_of_hosts_in_lan);

   sock = Inet_OpenRawSock(Options.netiface);

   Inet_GetIfaceInfo(Options.netiface, &MTU, NULL, NULL, NULL, NULL);

   buf = Inet_Forge_packet( ETH_HEADER + IP_HEADER + ICMP_HEADER );

   for (i=0; i<number_of_hosts_in_lan; i++)
   {
      if (inet_addr(Inet_MyIPAddress()) != inet_addr(Host_In_LAN[i].ip))
      {
         char MyMAC[6];
	 char DestMAC[6];

         Inet_GetMACfromString(Host_In_LAN[0].mac, MyMAC);
	 Inet_GetMACfromString(Host_In_LAN[i].mac, DestMAC);

         Inet_Forge_ethernet( buf, MyMAC, DestMAC, ETH_P_IP );

         Inet_Forge_ip( buf + ETH_HEADER,
                        inet_addr(Host_In_LAN[0].ip),
                        inet_addr(Host_In_LAN[i].ip),
                        sizeof(ICMP_header),
                        0xe77e,
                        0,
                        IPPROTO_ICMP );

         Inet_Forge_icmp( buf + ETH_HEADER + IP_HEADER, ICMP_ECHO, 0 );

         Inet_SendRawPacket(sock, buf, ETH_HEADER + IP_HEADER + ICMP_HEADER);
         usleep(1000);
      }
   }

   Inet_Forge_packet_destroy( buf );

#ifdef DEBUG
   Debug_msg("Inet_NoSniff -- after ICMP storm");
#endif

   buf = Inet_Forge_packet( MTU );

   fcntl(sock, F_SETFL, O_NONBLOCK);

   TIME_START;

   // Search for strange replies
   do
   {
      short pkttype;

      TIME_FINISH;

      len = Inet_GetRawPacket(sock, buf, MTU, &pkttype);

      if (len > 0 && pkttype == PACKET_HOST)
      {
         HEther = (ETH_header *) buf;
         if ( ntohs(HEther->type) == ETH_P_IP )
         {
            unsigned char MACS[20];
            HIP = (IP_header *)(HEther + 1);
            Inet_PutMACinString(MACS, HEther->source_mac);

            if (HIP->proto != IPPROTO_ICMP) continue;

#ifdef DEBUG
   Debug_msg("Inet_NoSniff -- got a ICMP reply after %.5f seconds", TIME_ELAPSED );
#endif

            for(i=0; i<number_of_hosts_in_lan; i++)
            {
               if ( inet_addr(Host_In_LAN[i].ip ) == HIP->source_ip )
               {
                  if (memcmp(MACS,Host_In_LAN[i].mac,17))
                  {
                     for (j=0; j<number_of_hosts_in_lan; j++)
                        if (!memcmp(MACS, Host_In_LAN[j].mac, 17)) break;

                     SniffTable[SniffTableIndex].Host_Index1=j;
                     SniffTable[SniffTableIndex].Host_Index2=i;
                     SniffTable[SniffTableIndex].mode=1;
                     SniffTableIndex++;
                     break;
                  }
               }
            }
         }
      }
   } while ( TIME_ELAPSED < 3 );

   // Search for strange ARP entries
   for (i=0; i<number_of_hosts_in_lan-1; i++)
      for(j=i+1; j<number_of_hosts_in_lan; j++)
         if (!memcmp(Host_In_LAN[i].mac, Host_In_LAN[j].mac, 17))
         {
            SniffTable[SniffTableIndex].Host_Index1=i;
            SniffTable[SniffTableIndex].Host_Index2=j;
            SniffTable[SniffTableIndex].mode=2;
            SniffTableIndex++;
         }

   Inet_Forge_packet_destroy( buf );

   close(sock);
   return (SniffTable);
}




void Inet_GetMACfromString(char *mac_string, unsigned char *MAC)
{
   unsigned int MAC_Add[6];
   int i;

   sscanf(mac_string,"%02X:%02X:%02X:%02X:%02X:%02X",
      (unsigned int *)&MAC_Add[0],(unsigned int *)&MAC_Add[1],(unsigned int *)&MAC_Add[2],
      (unsigned int *)&MAC_Add[3],(unsigned int *)&MAC_Add[4],(unsigned int *)&MAC_Add[5]);

   for (i=0; i<6; i++)
      MAC[i]=(unsigned char)MAC_Add[i];
}



void Inet_PutMACinString(char *mac_string, unsigned char *MAC)
{
   unsigned int MAC_Add[6];
   int i;

   for (i=0; i<6; i++)
      MAC_Add[i]=(unsigned int)MAC[i];

   sprintf(mac_string,"%02X:%02X:%02X:%02X:%02X:%02X",
      (unsigned int)MAC_Add[0],(unsigned int)MAC_Add[1],(unsigned int)MAC_Add[2],
      (unsigned int)MAC_Add[3],(unsigned int)MAC_Add[4],(unsigned int)MAC_Add[5]);

}

/* EOF */
