/*
    ettercap -- Inet -- Packet foundry

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

#include "include/ec_main.h"
#include "include/ec_inet_structures.h"
#include "include/ec_error.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

// protos....

u_char * Inet_Forge_packet( u_short size );
int Inet_Forge_packet_destroy( u_char *buf );
int Inet_Forge_ethernet( u_char *buf, u_char *source, u_char *dest, u_short type );
int Inet_Forge_arp( u_char *buf, u_short op, u_char *sa, u_long sip, u_char *da, u_long dip );
int Inet_Forge_ip( u_char *buf, u_long src, u_long dst, u_short len, u_short ident, u_short f_f, u_char proto );
int Inet_Forge_icmp( u_char *buf, u_char type, u_char code );
int Inet_Forge_tcp( u_char *buf, u_short sp, u_short dp, u_long seq, u_long ack, u_char flags, u_char *data, int data_len );
u_short Inet_Forge_ChecksumIP(u_short *buffer, int size);
u_short Inet_Forge_ChecksumTCP(u_short *buffer, u_short size, u_long IPsource, u_long IPdest);
void Inet_Forge_RemoveTimeStamp(char *opt, int len);

//---------------------------------------

u_char * Inet_Forge_packet( u_short size )
{

   u_char *buf;

   if (!size)
      size = MAX_PACKET_SIZE;

   buf = (u_char *)calloc(size, sizeof(u_char));
   if (buf == NULL)
      Error_msg("ec_Inet_Forge:%d calloc() | ERRNO : %d | %s", __LINE__, errno, sys_errlist[errno]);

   return (buf);
}


int Inet_Forge_packet_destroy( u_char *buf )
{

   if (buf != NULL) free(buf);

   return 0;

}


int Inet_Forge_ethernet( u_char *buf, u_char *source, u_char *dest, u_short type )
{
   ETH_header eth;

   memcpy(eth.source_mac, source, ETHER_ADDR_LEN);  // source address
   memcpy(eth.dest_mac, dest, ETHER_ADDR_LEN);      // destination address
   eth.type = htons(type);                          // packet type

   memcpy(buf, &eth, sizeof(eth));

   return (sizeof(eth));
}


int Inet_Forge_arp( u_char *buf, u_short op, u_char *sa, u_long sip, u_char *da, u_long dip )
{
   ARP_header arp;

   arp.hw_type       = htons(ARPHRD_ETHER);    // hardware address type
   arp.proto_type    = htons(ETH_P_IP);        // protocol address type
   arp.ha_len        = 6;                      // hardware address length
   arp.pa_len        = 4;                      // protocol address length
   arp.opcode        = htons(op);              // opcode
   memcpy(arp.source_add, sa, 6);              // source mac
   *(u_long *)arp.source_ip = sip;             // source ip
   memcpy(arp.dest_add, da, 6);                // dest mac
   *(u_long *)arp.dest_ip = dip;               // dest ip

   memcpy(buf, &arp, sizeof(arp));

   return (sizeof(arp));
}



int Inet_Forge_ip( u_char *buf, u_long src, u_long dst, u_short len, u_short ident, u_short f_f, u_char proto )
{
   IP_header ip;

   ip.h_len           = 5;                             // 20 byte header
   ip.version         = 4;                             // version 4
   ip.tos             = 0;                             // IP tos
   ip.t_len           = htons(IP_HEADER + len);        // total length
   ip.ident           = htons(ident);                  // IP ident
   ip.frag_and_flags  = htons(f_f);                    // fragmentation flags
   ip.ttl             = 125;                           // time to live
   ip.proto           = proto;                         // transport protocol
   ip.checksum        = 0;                             // checksum done in Inet_Forge_ChecksumIP
   ip.source_ip       = src;                           // source IP
   ip.dest_ip         = dst;                           // dest IP

   // checksumming...
   ip.checksum        = Inet_Forge_ChecksumIP( (unsigned short *)&ip, sizeof(IP_header) );

   memcpy(buf, &ip, sizeof(ip));

   return (sizeof(ip));
}


int Inet_Forge_icmp( u_char *buf, u_char type, u_char code )
{
   ICMP_header icmp;

   icmp.un.gateway  = 0;             // gateway address
   icmp.un.echo.seq = 1;             // seq
   icmp.un.echo.id  = htons(0xe77e); // ident   e77ercap :)
   icmp.code        = code;          // code
   icmp.type        = type;          // icmp type
   icmp.checksum    = 0;             // checksum

   // checksumming...
   icmp.checksum = Inet_Forge_ChecksumIP((u_short *)&icmp, sizeof(ICMP_header));

   memcpy(buf, &icmp, sizeof(icmp));

   return (sizeof(icmp));
}



int Inet_Forge_tcp( u_char *buf, u_short sp, u_short dp, u_long seq, u_long ack, u_char flags, u_char *data, int data_len )
{
   TCP_header *tcp;
   IP_header *ip;

   tcp = (TCP_header *)buf;
   tcp->source      = htons(sp);    // source port
   tcp->dest        = htons(dp);    // destination port
   tcp->seq         = htonl(seq);   // sequence number
   tcp->ack_seq     = htonl(ack);   // acknowledgement number
   tcp->flags       = flags;        // flags
   tcp->unused      = 0;            // UNUSED
   tcp->doff        = 5;            // 20 byte header
   tcp->window      = htons(32120); // window size
   tcp->checksum    = 0;            // checksum done in Inet_Forge_ChecksunTCP
   tcp->urg_ptr     = 0;            // urgent pointer


   if (data && data_len)
       memcpy(buf + TCP_HEADER, data, data_len);

   // checksumming...
   ip = (IP_header *)(buf - IP_HEADER);
   tcp->checksum    = Inet_Forge_ChecksumTCP( (u_short *)tcp, TCP_HEADER+data_len, ip->source_ip, ip->dest_ip );

   return (TCP_HEADER + data_len);
}



u_short Inet_Forge_ChecksumIP(u_short *buffer, int size)
{
   u_long cksum = 0;

   while(size >1)
   {
      cksum += *buffer++;
      size -= sizeof(u_short);
   }

   if (size) cksum += *(u_char*)buffer;

   cksum = (cksum >> 16) + (cksum & 0xffff);
   cksum += (cksum >> 16);
   return (u_short)(~cksum);

}



u_short Inet_Forge_ChecksumTCP(u_short *buffer, u_short size, u_long IPsource, u_long IPdest)
{
   u_long cksum = 0;
   u_short proto;
   u_short len;

   proto = htons(IPPROTO_TCP);
   len = size;

   while(size >1)
   {
      cksum += *buffer++;
      size -= sizeof(u_short);
   }

   cksum += IPsource >> 16;
   cksum += IPsource & 0xffff;
   cksum += IPdest >> 16;
   cksum += IPdest & 0xffff;
   cksum += proto;
   cksum += htons(len);

   if(size ) cksum += *(u_char*)buffer;

   cksum = (cksum >> 16) + (cksum & 0xffff);
   cksum += (cksum >> 16);
   return (u_short)(~cksum);

}


void Inet_Forge_RemoveTimeStamp(char *opt, int len)
{
    char *arrive;
    arrive = opt + len;
    while(opt < arrive)
    {
         if (*opt==8)
         {
            memset(opt,1,10);
            opt += 10;
         }
         else
            if (*opt == 1) opt++;
            else
               if (*opt > 1) opt += opt[1];
    }
}

/* EOF */
