/*
    ettercap -- a ncurses-based sniffer/interceptor utility for switched LAN

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>


#include "include/ec_main.h"
#include "include/ec_error.h"
#include "include/ec_inet.h"
#include "include/ec_simple.h"
#include "include/ec_signal.h"

#ifdef HAVE_GETOPT_H
    #include <getopt.h>
#endif

#ifdef HAVE_NCURSES
    #include "include/ec_interface.h"
#endif

#ifdef DEBUG
	#include "include/ec_debug.h"
#endif

// protos...

void Main_Usage(void);
void Main_Interactive(void);
void Main_Normal(void);
void Main_CheckForRun(void);

// global variables

char *program_argv0;

HOST *Host_In_LAN = NULL;			// ec_main.h
int number_of_hosts_in_lan;

CONNECTION *Conn_Between_Hosts = NULL;			// ec_main.h
int number_of_connections;

HOST Host_Source;
HOST Host_Dest;

char *Execute_Plugin;

OPTIONS Options;

int pipe_with_illithid;
int pipe_with_illithid_data;
int pipe_inject_stod[2];
int pipe_inject_dtos[2];


//-----------------------------------

int main(int argc, char *argv[])
{

	int c;

#ifdef HAVE_GETOPT_H
	static struct option long_options[] = {
		{ "help", no_argument, NULL, 'h' },
		{ "version", no_argument, NULL, 'v' },
		{ "simple", no_argument, NULL, 'N' },
		{ "list", no_argument, NULL, 'l' },
		{ "arpsniff", no_argument, NULL, 'a' },
		{ "sniff", no_argument, NULL, 's' },
		{ "macsniff", no_argument, NULL, 'm' },
		{ "first", required_argument, NULL, '1' },
		{ "second", required_argument, NULL, '2' },
		{ "interface", required_argument, NULL, 'i' },
		{ "check", no_argument, NULL, 'c' },
		{ "plugin", required_argument, NULL, 'p' },
		{ 0 , 0 , 0 , 0}
	};
#endif

	program_argv0 = argv[0];

#ifdef DEBUG
	Debug_Init();
	Debug_msg("Main");
#endif


	Signal_SigBuster();		// signal masking


#ifdef HAVE_GETOPT_H
	#ifdef PERMIT_PLUGINS
		while ((c = getopt_long (argc, argv, "hvNlasmc1:2:i:p:",long_options, (int *)0)) != EOF) {
	#else
		while ((c = getopt_long (argc, argv, "hvNlasmc1:2:i:",long_options, (int *)0)) != EOF) {		// no plugin
	#endif
#else
	#ifdef PERMIT_PLUGINS
		while ((c = getopt(argc, argv, "hvNlasmc1:2:i:p:")) != EOF) {
	#else
		while ((c = getopt(argc, argv, "hvNlasmc1:2:i:")) != EOF) {		// no plugin
	#endif
#endif
		switch (c) {

			case 'h':
				Main_Usage();
			break;

			case 'v':
				fprintf(stdout,"%s\n",VERSION);
				exit(0);
			break;

			case 'N':
				Options.normal = 1;
			break;

			case 'l':
				Options.list = 1;
			break;

			case 'a':
				Options.arpsniff = 1;
			break;

			case 's':
				Options.sniff = 1;
			break;

			case 'm':
				Options.macsniff = 1;
			break;

			case 'c':
				Options.check = 1;
			break;

#ifdef PERMIT_PLUGINS
			case 'p':
				Options.plugin = 1;
				Execute_Plugin = strdup(optarg);
			break;
#endif

			case '1':
				sscanf(optarg, "%128[^:]:%d", Host_Source.name, &Host_Source.port);
			break;

			case '2':
				sscanf(optarg, "%128[^:]:%d", Host_Dest.name, &Host_Dest.port);
			break;

			case 'i':
				strncpy(Options.netiface, optarg, 10);
			break;

			case ':': // missing parameter
				Main_Usage();
			break;

			case '?': // unknown option
				Main_Usage();
			break;
		}
	}

	Main_CheckForRun();		// it is ok ?

	if (!strcmp(Options.netiface, ""))			// set the default interface
		if (Inet_FindIFace(Options.netiface) == -1) exit(1);

	if ( Inet_CorrectIface(Options.netiface) < 0)
		Error_msg("Network Interface %s does NOT exist !!", Options.netiface);


	if (Options.normal)
		Main_Normal();
	else
		Main_Interactive();

return 0;
}




void Main_Usage(void)
{
	fprintf (stdout, "\n%s %s (c) 2000 %s\n", PROGRAM, VERSION, AUTHORS);
	fprintf (stdout, "\nUsage: %s [OPTION] ...\n\n", PROGRAM);

#if HAVE_GETOPT_H
	fprintf (stdout, "  -N, --simple                 NON interactive mode (without ncurses)\n");
	fprintf (stdout, "  -l, --list                   list all hosts in the lan\n");
	fprintf (stdout, "  -a, --arpsniff               arp based sniffing (you must use -1 and -2)\n");
	fprintf (stdout, "  -s, --sniff                  ip based sniffing (use -1 -2 to filter connections)\n");
	fprintf (stdout, "  -m, --macsniff               mac based sniffing (you must use -1 and -2)\n");
	fprintf (stdout, "  -1, --first HOST:PORT        source host:port (host:0 for all port)\n");
	fprintf (stdout, "  -2, --second HOST:PORT       dest host:port (host:0 for all port)\n");
	fprintf (stdout, "  -i, --interface IFACE        network interface to be used\n");
	fprintf (stdout, "  -c, --check                  check for other poisoners in the LAN\n");
#ifdef PERMIT_PLUGINS
	fprintf (stdout, "  -p, --plugin [name]          run the plugin (\"list\" for available ones)\n");
#endif
	fprintf (stdout, "  -v, --version                print version\n");
	fprintf (stdout, "  -h, --help                   print this help\n");
	fprintf (stdout, "\n");
#else
	fprintf (stdout, "  -N                   NON interactive mode (without ncurses)\n");
	fprintf (stdout, "  -l                   list all hosts in the lan\n");
	fprintf (stdout, "  -a                   arp based sniffing (you must use -1 and -2)\n");
	fprintf (stdout, "  -s                   ip based sniffing (use -1 -2 to filter connections)\n");
	fprintf (stdout, "  -m                   mac based sniffing (you must use -1 and -2)\n");
	fprintf (stdout, "  -1 HOST:PORT         source host:port (host:0 for all port)\n");
	fprintf (stdout, "  -2 HOST:PORT         dest host:port (host:0 for all port)\n");
	fprintf (stdout, "  -i IFACE             network interface to be used\n");
	fprintf (stdout, "  -c                   check for other poisoners in the LAN\n");
#ifdef PERMIT_PLUGINS
	fprintf (stdout, "  -p [name]            run the plugin (\"list\" for available ones)\n");
#endif
	fprintf (stdout, "  -v                   print version\n");
	fprintf (stdout, "  -h                   print this help\n");
	fprintf (stdout, "\n");
#endif

	exit (0);
}



void Main_Interactive(void)
{
#ifdef HAVE_NCURSES
	struct winsize  ws = {0, 0, 0, 0};
#endif

#ifdef DEBUG
	Debug_msg("Main_Interactive");
#endif

#ifdef HAVE_NCURSES

	ioctl(0, TIOCGWINSZ, &ws);				// syscall for the window size

	if ( (ws.ws_row < 25) || (ws.ws_col < 80) )
		Error_msg("Screen must be at least 25x80 !!");

   memset(&Host_Source, 0, sizeof(HOST) );
   memset(&Host_Dest, 0, sizeof(HOST) );

	printf("Building host list for netmask %s, please wait...\n", Inet_MySubnet());
	number_of_hosts_in_lan = Inet_HostInLAN();

	Interface_InitTitle(Host_In_LAN[0].ip, Host_In_LAN[0].mac, Inet_MySubnet());
	Interface_InitScreen();
	Interface_Run();
#else
	#ifdef DEBUG
		Debug_msg("Ncurses not supported -- turning to non interactive mode...");
	#endif
	Main_Normal();
#endif

#ifdef DEBUG
	Debug_msg("Main_Interactive_END");
#endif
}




void Main_Normal(void)
{
#ifdef DEBUG
	Debug_msg("Main_Normal");
#endif

	printf ("\n\033[01m\033[1m%s %s (c) 2000 %s\033[0m\n\n", PROGRAM, VERSION, AUTHORS);
	printf ("Your IP: %s with MAC: %s on Iface: %s\n", Inet_MyIPAddress(), Inet_MyMACAddress(), Options.netiface);


	if (Options.list || Options.check || Options.arpsniff || Options.sniff  || Options.macsniff)
	{
		printf("Building host list for netmask %s, please wait...\n", Inet_MySubnet());
		number_of_hosts_in_lan = Inet_HostInLAN();
	}

	if (Options.list)
		Simple_HostList();

	if (Options.check)
		Simple_CheckForPoisoner();

#ifdef PERMIT_PLUGINS
	if (Options.plugin)
		Simple_Plugin();
#endif

	if (Options.arpsniff || Options.sniff  || Options.macsniff)
		Simple_Run();

	printf("\n");
#ifdef DEBUG
	Debug_msg("Main_Normal_END");
#endif

exit(0);
}



void Main_CheckForRun(void)
{

#ifdef DEBUG
	Debug_msg("Main_CheckForRun");
#endif

#ifdef PERMIT_SUID
	if (geteuid() != 0)
		Error_msg("Sorry EUID %d, must be EUID = 0 to run %s !!", geteuid(), PROGRAM);
#else
	if (getuid() != 0)
		Error_msg("Sorry UID %d, must be root to run %s !!", getuid(), PROGRAM);
#endif


	if ( (Options.arpsniff + Options.sniff + Options.macsniff) > 1 )
		Error_msg("Please select only one sniffing method !!\n\n");

	if (Options.arpsniff || Options.macsniff )
		if ( (!strcmp(Host_Source.name, "")) || (!strcmp(Host_Dest.name, "")) )
		{
			#ifdef DEBUG
				Debug_msg("Main_CheckForRun -- 1) [%s][%d] 2) [%s][%d] ", Host_Source.name, Host_Source.port, Host_Dest.name, Host_Dest.port);
			#endif
			Error_msg("Please specify first (-1) AND second (-2) host !!");
		}
}

/* EOF */
