/*
    ettercap -- PlugIns module

    Copyright (C) 2001  ALoR <alor@thepentagon.com>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "include/ec_main.h"

#ifdef PERMIT_PLUGINS   // don't compile if plug-in not supported

#include <dlfcn.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <dirent.h>
#include <sys/time.h>

#include "include/ec_plugins.h"

#ifdef HAVE_NCURSES
   #include <ncurses.h>
   #include "include/ec_interface_plugins.h"
   extern WINDOW *plugin_window;
#endif

#ifdef DEBUG
   #include "include/ec_debug.h"
#endif


PLUGINS *Plugins_list = NULL;

// protos....

void Plugin_RunPlugIn(char *name, char **argv);
int Plugin_Input(char *string, short size, short mode);
void Plugin_Output(char *message, ...);
void Plugin_SYS_Output(char *message, ...);

int Plugin_MakePlugList(void);
char *Plugin_Getname(char *plugin);

// --------------------------

void Plugin_RunPlugIn(char *name, char **argv)
{
   static void *handle;
   char *error;
   void (*Start)(char **);

#ifdef DEBUG
   Debug_msg("Plugin_RunPlugIn -- starting %s plugin", name);
#endif
#ifdef HAVE_NCURSES
	if (!Options.normal) echo();
#endif

   Plugin_SYS_Output("Starting %s plugin...", name);

   handle = dlopen(name, RTLD_NOW | RTLD_GLOBAL);
   if (!handle)
   {
      #ifdef DEBUG
         Debug_msg("Plugin_RunPlugIn -- dlopen() | %s", dlerror());
      #endif
      Plugin_SYS_Output("Error Opening %s... (dlopen)", name);
      return;
   }

   Start = dlsym(handle, "PlugIn_Start");
   if ((error = dlerror()) != NULL)
   {
      #ifdef DEBUG
         Debug_msg("Plugin_RunPlugIn -- dlsym() | %s", error);
      #endif
      Plugin_SYS_Output("Error Opening %s... (dlsym)", name);
      return;
   }

   (*Start)(argv);   // lounch the plugin

   Plugin_SYS_Output("%s plugin ended.", name);

   dlclose(handle);

#ifdef DEBUG
   Debug_msg("Plugin_RunPlugIn -- shutting down %s plugin", name);
#endif
#ifdef HAVE_NCURSES
	if (!Options.normal) noecho();
#endif

}


int Plugin_Input(char *string, short size, short mode)
{

	fd_set msk_fd;
	struct timeval TimeOut;

	if (mode == P_BLOCK)
	{
		#ifdef HAVE_NCURSES
   		if (!Options.normal)	{ wnoutrefresh(plugin_window); doupdate(); }
   	#endif
      if (size == 1)
      {
       	#ifdef HAVE_NCURSES
   			if (!Options.normal)
					string[0] = wgetch(plugin_window);
				else
			#endif
      		string[0] = getchar();
			fflush(stdin);
      	return 1;
      }
      else
      {
      	#ifdef HAVE_NCURSES
   			if (!Options.normal)
					wgetnstr(plugin_window, string, size);
				else
			#endif
					scanf("%s", string);
			fflush(stdin);
			return strlen(string);
		}
	}
	else // P_NONBLOCK
	{
		memset(&TimeOut, 0, sizeof(TimeOut));	//  timeout = 0
		FD_ZERO(&msk_fd);
		FD_SET(0, &msk_fd);

		select(FOPEN_MAX, &msk_fd, (fd_set *) 0, (fd_set *) 0, &TimeOut);

		if (FD_ISSET(0, &msk_fd))
		{
			#ifdef HAVE_NCURSES
   			if (!Options.normal) { wnoutrefresh(plugin_window); doupdate(); }
   	   #endif
      	if (size == 1)
      	{
       		#ifdef HAVE_NCURSES
   				if (!Options.normal)
						string[0] = wgetch(plugin_window);
					else
				#endif
      			string[0] = getchar();
      		fflush(stdin);
      		return 1;
      	}
      	else
      	{
      		#ifdef HAVE_NCURSES
   				if (!Options.normal)
						wgetnstr(plugin_window, string, size);
					else
				#endif
					scanf("%s", string);
				fflush(stdin);
				return strlen(string);
			}
		}
		else
		{
			usleep(5);
			return 0;
		}
	}

}



void Plugin_Output(char *message, ...)
{
   va_list ap;
   char plug_output[250];  // should be enough


#ifndef HAVE_VPRINTF
#errors "Must have vsprintf()."
#endif

   va_start(ap, message);
   vsnprintf(plug_output, 250, message, ap);
   va_end(ap);

#ifdef HAVE_NCURSES
   if (!Options.normal)
   {
      wprintw(plugin_window, "%s", plug_output);
      wnoutrefresh(plugin_window);
      doupdate();
   }
   else
#endif
	{
   	fprintf(stdout, "%s", plug_output);
   	fflush(stdout);
   }

}


void Plugin_SYS_Output(char *message, ...)
{
   va_list ap;
   char plug_output[250];  // should be enough


#ifndef HAVE_VPRINTF
#errors "Must have vsprintf()."
#endif

   va_start(ap, message);
   vsnprintf(plug_output, 250, message, ap);
   va_end(ap);


#ifdef HAVE_NCURSES
   if (!Options.normal)
   {
      #define HELP_COLOR 6
      #define NORM_COLOR 1
      wbkgdset(plugin_window, COLOR_PAIR(HELP_COLOR)); wattron(plugin_window, A_BOLD);
      wprintw(plugin_window, "\n%s\n", plug_output);
      wbkgdset(plugin_window, COLOR_PAIR(NORM_COLOR)); wattroff(plugin_window, A_BOLD);
      wnoutrefresh(plugin_window);
      doupdate();
   }
   else
#endif
	{
   	fprintf(stdout, "\n\033[36m %s \033[0m\n", plug_output);
		fflush(stdout);
	}

}



int Plugin_MakePlugList(void)
{
   struct dirent **namelist;
   static void *handle;
   char name[100];
   char *Plug_Info;
   short *Plug_Ver;
   short i, k, j=0;

   #include "include/ec_plugins_path.h"   // char *path = "...";
	#define MAX_PLUGIN 100

#ifdef DEBUG
      Debug_msg("Plugin_MakePlugList -- %s", path);
#endif

	Plugins_list = (PLUGINS *)calloc(MAX_PLUGIN, sizeof(PLUGINS));

   for(k=0; k<2; k++)
   {
   	short n=0, m=0;

      if (k)
         m = scandir(path, &namelist, 0, alphasort);
      else
         n = scandir(".", &namelist, 0, alphasort);

      if ( (n > 0) || (m > 0) )
      {
         short num;
         if (k) num = m;
         else num = n;

          for(i=0; i<num; i++)
            if ( strstr(namelist[i]->d_name, "ec_") && strstr(namelist[i]->d_name, ".so") )
            {
               Plugins_list[j].name = strdup(namelist[i]->d_name);

               if (k) strcpy(name, path);
               else strcpy(name, "./");

               Plugins_list[j].path = strdup(name);

               strcat(name, namelist[i]->d_name);

               #ifdef DEBUG
                  Debug_msg("\t %s  %s", Plugins_list[j].path, Plugins_list[j].name );
               #endif

               handle = dlopen(name, RTLD_NOW | RTLD_GLOBAL);
               if (handle)
               {
               	char *symerror;

                  Plug_Info = dlsym(handle, "PlugIn_Info");

   					if (( symerror = dlerror()) != NULL)
   						Plugins_list[j].description = strdup("No info available...");
						else
 	               	Plugins_list[j].description = strdup(Plug_Info);

   					Plug_Ver = dlsym(handle, "PlugIn_Version");

   					if (( symerror = dlerror()) != NULL)
   						Plugins_list[j].version = 0;
						else
 	               	Plugins_list[j].version = (float)(*Plug_Ver)/10;

                	dlclose(handle);
               }
               else
               {
               	Plugins_list[j].version = -1;
                  Plugins_list[j].description = strdup("Seems to be an invalid plugin...");
               }

               j++;
               if (j == MAX_PLUGIN) return j;
            }
      }
   }

   return j;

}

char *Plugin_Getname(char *plugin)
{
	char *name;

	name = strdup(plugin);
	name[strlen(name)-3] = 0;

	return name+3;
}


#endif   // PERMIT_PLUGINS

/* EOF */
